#!/usr/bin/perl

#########################################################################
#                                                                       #
# Copyright and Intellectual Property Right Notice:                     #
#                                                                       #
# All contents of this Source Code are: Copyright  2000-2005            #
# FRISK Software International (FRISK). All rights reserved.            #
#                                                                       #
# FRISK retains all title and ownership to the Source Code and any      #
# derivative works. FRISK owns copyrights, inventions, know-how and     #
# trade secrets related to the design, operation, amendments or         #
# services of the Source Code and any derivative works which are        #
# developed from the Source Code. FRISK retains all rights to           #
# modify, prepare derivative works of, decompile, reverse engineer,     #
# and disassemble the Source Code from the Software.                    #
#                                                                       #
# Use and modification of the Source Code shall only be permitted by    #
# any licensed party pursuant a prior written authorization from        #
# FRISK.                                                                #
#                                                                       #
#########################################################################

############################  NOTES #####################################
#                                                                       #
# What this script does:                                                #
# *  Sets up symlinks to executables and man pages so they'll work      #
#    without needing full paths each time (All editions), and creates   #
#    the link /etc/f-prot.conf -> $INSTALLDIR/etc/f-prot.conf.default   #
#    if no such link exists.
# *  Sets up rc-scripts for f-protd (FS and MS editions) and scan-mail  #
#    (MS edition) if we find suitable files in tools/rc_scripts/.       #
#    It modifies the rc scripts on the fly, changing the 'BASEPATH='    #
#    variable to hold the installation directory. (If you create a      #
#    working rc script for your distro, feel free to share ;)           #
# *  (MS edition only) Offers to configure the system for email         #
#    scanning via Sendmail, Postfix or Qmail. Instructions on how to do #
#    this manually can be found in the documentation that should've     #
#    accompanied this package.  Note that when determining if f-prot    #
#    has already been configured, the program simply searches for the   #
#    regexp /^# F-Prot/ in the relevant config files, so it's very      #
#    important that the comments it creates in those files are not      #
#    removed. Otherwise, it'll append the same config lines again.      #
#                                                                       #
#########################################################################

use strict;
use Cwd;

#Do we have File::Copy?
my $copyfunction;
eval{ use File::Copy; };
if($@){
	$copyfunction = sub {
		open(IN, "<$_[0]") || return 0;
		open(OUT, ">$_[1]") || return 0;
		print OUT <IN>;
		close(IN);
		close(OUT);
		return 1;
	};
}else{
	$copyfunction =\&copy;
}

my $eexist = 17; #usually;
eval{ use POSIX qw(:errno_h); };
if(! $@){ $eexist = EEXIST; }

# This hash holds all variables/info needed to install
my %Config = (
	#platform info
	OS		=> undef,
	distro	=> undef,
	release	=> undef,
	march	=> undef,
	#f-prot flavor
	edition	=> "Workstation", # untill we find out otherwise
	installdir => undef,
	#linklocations
	bin		=> undef,
	sbin	=> undef,
	man1	=> undef,
	man5	=> undef,
	man8	=> undef,
	marker	=> "# F-Prot - Please leave this line in place" # if this changes, change the regexp that searches for it too.
);

sub main()
{
	#gather config and welcome the user
	fillConfigInfo();
	printf("\n\n\t(c) FRISK Software International\n\n\thttp://www.f-prot.com/\n\n");
	printf("\tYou are about to install F-Prot Antivirus for %s %ss\n ", $Config{OS}, $Config{edition});
	printf("\ton a %s%s %s running on %s at '%s'\n\n", $Config{distro}, $Config{OS}, $Config{release}, $Config{march}, $Config{installdir});

	#non-default installdir?
	if( $Config{installdir} ne "/usr/local/f-prot"){
		print "\tBe warned that the documentation and user manuals assume that\n";
		print "\tF-Prot is installed in the default directory '/usr/local/f-prot'.\n\n";
		print "\tF-Prot will run just fine from '" . $Config{installdir} . "'\n";
		print "\tbut you will of course have to adjust sample configuration and\n";
		print "\tsuch to match that change.\n\n";
	}

	my $configfile = "$Config{installdir}/etc/f-prot.conf.default";
	if( ! -e $configfile){
		die("\n\nERROR: $configfile is missing.\n\n This means that:\n\
a)\tThis script is being used with the wrong version of f-prot,\n\
b)\tThe F-Prot package is damaged or incomplete.\n\
c)\tThis script is not being run from the top-level f-prot directory\n\nRe-run this program when this issue is resolved.\n\n");
	}
	makeLink("/etc/f-prot.conf", $configfile);

	makeSymlinks();
	changePermissions();
	print "\nUpdating virus signature files ...\n";
	if( 0 != system($Config{installdir} . "/tools/check-updates.pl")){
		die("\n\tUpdate check failed. That indicates something is seriously wrong.\n\nExiting\n");
	}
	setupRCfiles();
	setupConfig();

	print "\n\n\tAll done!\n\n";
	print "\n\nIf you reconfigured your MTA you should restart it now to activate the changes.\n\n";
	print "\tHave a nice day\n\nFrisk software (www.f-prot.com)\n\n";
}


sub changePermissions
{
	print "\nSetting file permissions ...";
	chmod(0755, $Config{installdir} . "/f-prot") || warn "failed\n$Config{installdir}/f-prot may have wrong file permissions\n";
	chmod(0700, $Config{installdir} . "/tools/check-updates.pl") || warn "failed\n$Config{installdir}/tools/check-updates.pl\n";
	if( $Config{edition} eq "Workstation" ){
		return;
	}
	chmod(0755, $Config{installdir} . "/f-protd") || warn "failed\n$Config{installdir}/f-protd may have wrong file permissions\n";
	if( $Config{edition} eq "FileServer" ){
		return;
	}
	chmod(0755, $Config{installdir} . "/tools/scan-mail.pl") || warn "failed\n$Config{installdir}/tools/scan-mail.pl may have wrong file permissions\n";
	if( $Config{OS} =~ /aix/i ){ # we dont ship qmail and milter stuff with aix versions
		return;
	}
	chmod(0755, $Config{installdir} . "/tools/f-prot-milter") || warn "failed\n$Config{installdir}/tools/f-prot-milter may have wrong file permissions\n";
	chmod(0755, $Config{installdir} . "/tools/f-prot-qmail") || warn "failed\n$Config{installdir}/tools/f-prot-qmail may have wrong file permissions\n";
	chmod(0777, $Config{installdir} . "/backup") || warn "failed\n$Config{installdir}/backup not be writeable by your mta user\n";
}

sub setupConfig()
{
	my $reply;
	if( $Config{edition} eq "Workstation"){
		return;
	}
	# start f-protd now?
	my $fpcmd;
	if( ($reply = answerMeThis("Would you like to start f-protd now", "Y/n")) =~ /^y/i ){
		if( -e "/etc/init.d/f-protd" ){
			$fpcmd = "/etc/init.d/f-protd start";
		}else{
			$fpcmd = $Config{installdir} . "/f-protd --background";
		}
		print "Executing '$fpcmd'\n";
		if( 0 != system($fpcmd) ){
			die("\n\nf-protd failed to start. Please investigate and then re-run this program.\n\n");
		}
	}
	# set up for auto-start?
	if( -e "/etc/init.d/f-protd" ){
		if($Config{OS} =~ /linux/i && ( $Config{distro} =~ /suse|redhat|mandrake|debian/i )){
			$reply = answerMeThis("Would you like f-protd to be started automatically upon system boot", "Y/n");
			my $command = undef;
			if($reply =~ /^y/i){
				if(     $Config{distro} =~ /redhat/i){
					$command = "chkconfig --add f-protd";
				}elsif( $Config{distro} =~ /suse/i){
					$command = "chkconfig f-protd on";
				}elsif( $Config{distro} =~ /mandrake/i){
					$command = "chkconfig --add f-protd start";
				}elsif( $Config{distro} =~ /debian/i){
					$command = "update-rc.d f-protd start 12 2 3 4 5 . stop 21 0 1 6 .";
				}
				if( 0 != system($command) ){
					print "\n\nchkconfig failed! F-protd will NOT be started automatically on the next reboot.";
					print "\nInstallation continues, but you should investigate this matter.\n";
				}
			}
		}else{
			print "\nDon't know how this OS/distro handles boot scripts.";
			print "\nIf you want f-protd to be started automatically upon system boot,";
			print "\nyou'll have to make that happen yourself.";
		}
	}
	if($Config{edition} eq "FileServer"){
		return;
	}
	#Configure scan-mail
	print "\n\n####################################################################################\n\n";
	print "\tIf you use Sendmail, Postfix or QMail as your MTA, this program can\n";
	print "\tconfigure them to scan incoming emails using f-prot and scan-mail.pl.";
	print "\n\n####################################################################################\n\n";

	if( (my $reply = answerMeThis("Do you want the installation program to add virus scanning to your MTA setup", "Y/n")) !~ /^y/i){
		print "\nvery well. Not configuring mailservers at this point.\n\n\tThe 'SCANMAIL_STARTUP' variable in /etc/f-prot.conf may need manual adjustments.\n";
		return;
	}else{
		doScanmailConfig();
		if( $Config{OS} =~ /linux/i && $Config{distro} =~ /suse|redhat|debian/i){
			if(($reply = answerMeThis("Do you want scan-mail.pl to be started automatically upon system boot", "Y/n")) =~ /^y/i){
				my $command = undef;
				if($Config{distro} =~ /redhat/i){
					$command = "chkconfig --add scan-mail";
				}elsif( $Config{distro} =~ /suse/i){
					$command = "chkconfig scan-mail on";
				}elsif( $Config{distro} =~ /debian/i){
					$command = "update-rc.d scan-mail start 12 2 3 4 5 . stop 21 0 1 6 .";
				}

				if( 0 != system($command) ){
					print "\n\nchkconfig failed! F-protd will NOT be started automatically on the next reboot.";
					print "\nPlease investigate this matter.\n";
				}
			}
		}
		else{
			print "\nDon't know how this OS/distro handles boot scripts.";
			print "\nIf you want f-protd to be started automatically upon system boot,";
			print "\nyou'll have to make that happen yourself.";
		}
	}
}

sub doScanmailConfig()
{
	print "\nScan-mail.pl needs to know which mailserver you use.\n";
	print "\n 1 - Qmail\t Configure scan-mail.pl for use with f-prot-qmail";
	print "\n 2 - Postfix\t Configure scan-mail.pl for use with Postfix";
	print "\n 3 - Sendmail\t Configure scan-mail for use with Sendmail/milter";
	print "\n";
	my $args = "";
	my $reply;
	while( ($reply = answerMeThis("Which mail server do you want scan-mail.pl to service", "")) !~ /^(1|2|3)$/){
		print "\n Invalid reply '$reply'. Try again\n";
	}
	if( $reply    == 1){
		$args = doQmailStuff();
	}elsif($reply == 2){
		$args = doPostfixStuff();
	}elsif($reply == 3){
		$args = doSendmailStuff();
	}
	if($args eq ""){
		return;  # user decided not to configure mta, so we don't touch the config file
	}
	open(CONF, "</etc/f-prot.conf") || die("Cannot open /etc/f-prot.conf: $!\n Cannot continue\n\n");
	my @conf = <CONF>;
	close(CONF);
	foreach (@conf){
		s/^\s*SCANMAIL_STARTUP\s*=.*/SCANMAIL_STARTUP=\"$args\"\n/;
	}
	if( ! grep("SCANMAIL_STARTUP", @conf)){
		push @conf, "SCANMAIL_STARTUP=\"$args\"\n";
	}
	open(CONF, ">/etc/f-prot.conf") || die("Cannot open /etc/f-prot.conf: $!\n Cannot continue\n\n");
	print CONF @conf;
	close(CONF);
	if( ($reply = answerMeThis("Would you like to start scan-mail.pl now", "Y/n")) =~ /^y/i){
		my $smcmd;
		if( -e "/etc/init.d/scan-mail" ){
			$smcmd = "/etc/init.d/scan-mail start";
		}else{
			$smcmd = $Config{installdir} . "/tools/scan-mail.pl $args";
			print "\nFrom now on, scan-mail.pl should be started with the following command:";
			print "\n'$smcmd'";
			print "\n\nWe recommend you put this in a script or make other arrangements so";
			print "\nscan-mail.pl gets started automatically upon system boot.\n\n";
		}
		print "Executing '$smcmd'\n";
		if( 0 != system($smcmd) ){
			print "\n\nscan-mail.pl failed to start. Please investigate...\n\n";
		}
	}
}

sub doQmailStuff()
{
	my $ret = "--daemon";
	print "\n\n\tTo use f-prot with qmail, f-prot-qmail must take the place of the qmail-queue program";
	print "\n\tand qmail-queue must be renamed to qmail-queue.orig and both must have correct";
	print "\n\townership and permissions.\n";
	my $reply = answerMeThis("Do you wish to make these changes now", "Y/n");
	if( $reply !~ /^y/i ){
		print "\nVery well. Instructions on how to do this manually can be found in the accompanying documentation.\n";
		return $ret;
	}
	# following code is critical. It must make sure the system is always in a usable state.
	# I.e. it should be considered atomic from the users point of view
	my $qqueue = getPathToExistingFile("qmail-queue", 1, "/var/qmail/bin/qmail-queue");
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($qqueue);
	my $tempfile = $qqueue . ".f-prot";
	print "\npreparing f-prot-qmail";
	if( ! &$copyfunction("$Config{installdir}/tools/f-prot-qmail", $tempfile) ){
		die("\nFailed to copy f-prot-qmail to $tempfile\n". \
			"\nNo changes have been made to your mailserver setup yet." . \
			"\nPlease investigate and fix the above error and then re-run this script\n");
	}
	if( ! chown($uid, $gid, $tempfile) ){
		unlink($tempfile);
		die("\n chown() on $tempfile failed: $!\n" . \
			"\nNo changes have been made to your mailserver setup yet." . \
			"\nPlease investigate and fix the above error and then re-run this script\n");
	}
	if( ! chmod($mode, $tempfile) ){
		unlink($tempfile);
		die("\n chmod() on $tempfile failed: $!\n". \
			"\nNo changes have been made to your mailserver setup yet." . \
			"\nPlease investigate and fix the above error and then re-run this script\n");
	}
	print "\nswitching binaries...";
	if( -e $qqueue . ".orig" ){
		#Propably an upgrade or a re-run of this script. Either way, remove qmail-queue, but dont overwrite
		#qmail-queue.orig
		print "\n ...what's this?\n\n ...qmail-queue.orig already exists. I'll leave it alone...";
		if( ! unlink($qqueue) ){
			unlink($tempfile);
			die("\n unlink() on previous $qqueue failed: $!\n" . \
				"\nNo changes have been made to your mailserver setup yet." . \
				"\nPlease investigate and fix the above error and then re-run this script\n");
		}
	} else {
		if( ! rename($qqueue, $qqueue . ".orig") ){
			unlink($tempfile);
			die("\nCannot rename $qqueue to $qqueue.orig: $!\n " . \
				"\nNo changes have been made to your mailserver setup yet." . \
				"\nPlease investigate and fix the above error and then re-run this script\n");
		}
	}
	if( ! rename($tempfile, $qqueue) ){
		print "Cannot rename $qqueue to $qqueue.orig: $!\n ";
		# Try to back out.
		if( ! rename($qqueue . ".orig", $qqueue) ){
			print "\n\tERROR: Failed to roll back changes. Your mailserver setup may not be able to deliver mail";
			print "\nuntill qmail-queue.orig is renamed back to qmail-queue with";
			printf("\nuid %d, gid %d and permission bits set to %o\n", $uid, $gid, $mode);
		}else{
			print "\nNo changes have been made to your mailserver setup yet.";
			print "\nPlease investigate and fix the above error and then re-run this script\n";
		}
		unlink($tempfile);
		die("\nExiting\n");
	}
	print " done\n";

	return $ret;
}

sub doPostfixStuff()
{
	print "\n\n\tTo enable email-scanning with Postfix mailservers, Postfix's configuration";
	print "\n\tfiles need to be modified.\n";
	print "\n\tIf you are upgrading from a previous version and already have a working";
	print "\n\tconfiguration in place, you should skip this step.\n";
	print "\n\tIf you are trying to repair a broken configuration, then remove all f-prot";
	print "\n\trelated config lines from both master.cf and main.cf before proceeding.\n\n";

	my $reply = answerMeThis("Do you wish to make the changes now", "Y/n");
	if($reply !~ /^y/i ){
		print "\nOperation cancelled. Refer to scan-mail.pl's man page for instructions on how to\n";
		print "do this manually. Furthermore, you must remember to update the 'SCANMAIL_STARTUP' variable\n";
		print "in /etc/f-prot.conf at the same time if you plan to use a rc-script to start scan-mail.pl\n\n";
		return "";
	}
	my $ret = "";
	# find config files
	my $maincf = getPathToExistingFile("main.cf", 1, "/etc/postfix/main.cf");
	my $mastercf = $maincf;
	$mastercf =~ s/main.cf/master.cf/;
	$mastercf = getPathToExistingFile("master.cf", 1 , $mastercf);
	backupFile($maincf);
	backupFile($mastercf);
	my $mainline = "\n$Config{marker}\ncontent_filter = smtp:[127.0.0.1]:10025\n";
	my $masterlines = "\n$Config{marker}\nscan unix - - n - 10 smtp\n";
	$masterlines .= "localhost:10026 inet n - n - 10 smtpd\n";
	$masterlines .= "\t-o content_filter= \n";
	$masterlines .= "\t-o local_recipient_maps= \n";
	$masterlines .= "\t-o relay_recipient_maps= \n";
	$masterlines .= "\t-o myhostname=localhost \n";
	$masterlines .= "\t-o smtpd_helo_restrictions= \n";
	$masterlines .= "\t-o smtpd_client_restrictions= \n";
	$masterlines .= "\t-o smtpd_sender_restrictions= \n";
	$masterlines .= "\t-o smtpd_recipient_restrictions=permit_mynetworks,reject \n";
	$masterlines .= "\t-o mynetworks=127.0.0.0/8 \n";

	print "\n\n\tScan-mail.pl can act as a mail filter with postfix in 2 ways;";
	print "\neither as a spawn service, where scan-mail.pl is started for each request";
	print "\nor as a stand-alone smtp proxy running in the background.\n";
	print "\n\t See 'man scan-mail.pl' for details\n";
	print "\n\tChoose one:\n";
	print "\n 1) \tSpawn service";
	print "\n 2) \tStandalone SMTP proxy\n";
	while(($reply = answerMeThis("Which mode would you like scan-mail.pl to operate in", "")) !~ /^(1|2)$/){
		print "\nInvalid reply '$reply'.\n\tChoose either '1' or '2':\n";
	}
	if($reply == '1'){
		print "\nSpawn service it is\n";
		$masterlines .= "localhost:10025 inet n n n - 10 spawn user=nobody argv=$Config{'installdir'}/tools/scan-mail.pl -postfix 127.0.0.1:10026";
		$ret = "";
	}else{
		print "\nStandalone smtp proxy it'll be then\n";
		$ret = "--daemon --proxy --proxy_address 127.0.0.1:10025 --smtp_address 127.0.0.1:10026";
	}

	# Let's make the changes
	print "\nUpdating $maincf ...";
	# first we check if we've already made the necessary changes in a previous run perhaps
	open(MAIN, "<$maincf") || die("Cannot open $maincf for reading: $!\nPlease investigate and then rerun this script\n");
	my @m = <MAIN>;
	close(MAIN);
	if( scalar grep(/$Config{'marker'}/, @m)){
		print "\n\n\tPrevious F-Prot configuration section found in $maincf";
		print "\n\tLeaving configuration intact\n";
	}else{
		open(MAIN, ">>$maincf") || die("Cannot open $maincf for updates: $!\nPlease investigate and then rerun this script\n");
		print MAIN $mainline;
		close(MAIN);
		print " done\n";
	}
	print "\nUpdating $mastercf ...";
	# again, we first check if we've already made the necessary changes
	open(MASTER, "<$mastercf") || die("Cannot open $mastercf for reading: $!\nPlease investigate and then rerun this script\n");
	my @m = <MASTER>;
	close(MASTER);
	if( scalar grep(/$Config{'marker'}/, @m)){
		print "\n\n\tPrevious F-Prot configuration section found in $mastercf";
		print "\n\tLeaving configuration intact\n";
	}else{
		open(MASTER, ">>$mastercf") || die("Cannot open $mastercf for updates: $!\nPlease investigate and then rerun this script\n");
		print MASTER $masterlines;
		close(MASTER);
		print " done\n";
	}
	return $ret;
}

sub doSendmailStuff()
{
	print "\n\n\tFor F-Prot to work with Sendmail, Sendmail must have been";
	print "\n\tcompiled with milter support. Practically all modern versions of";
	print "\n\tSendmail have milter support, so I'll assume we're good on that front";
	print "\n\n\tTo enable f-prot-milter, Sendmail's configuration must be changed a bit.\n";
	my $reply = answerMeThis("Do you wish to make those changes now", "Y/n");
	if( $reply !~ /^y/i ){
		print "\nOperation cancelled. Refer to the man pages for scan-mail.pl and f-prot-milter";
		print "\nfor instructions on how to do this manually. ";
		print "\nFurthermore, you must remember to update /etc/f-prot.conf at the\n";
		print "same time if you plan to use a rc-script to start scan-mail.pl\n\n";
		return "";
	}
	my $cflines = "\n$Config{marker}\nO InputMailFilters=f-prot-milter\n";
	$cflines .= "Xf-prot-milter, S=inet:12200\@127.0.0.1, F=T,T=E:8m;C:5m;R:4m;S:2m\n";
	my $mclines = "\n$Config{marker}\ndefine(`_FFR_MILTER', `True')\n";
	$mclines .= "INPUT_MAIL_FILTER(`f-prot-milter', `S=inet:12200\@127.0.0.1, F=T, T=E:8m;C:5m;R:4m;S:2m')\n";
	# Find config file
	print "\nPlease enter the path to your sendmail configuration file, either the .mc file or the .cf file\n";
	my $default = "";
	if( -e "/etc/mail/sendmail.mc" ){
		$default = "/etc/mail/sendmail.mc";
	}elsif( -e "/etc/mail/sendmail.cf" ){
		$default = "/etc/mail/sendmail.cf";
	}
	my $sendmailc = getPathToExistingFile("sendmail.(mc|cf)", 1, $default); # TESTME
	#Make a backup copy
	backupFile($sendmailc);
	my $ret = "--daemon --milter ";
	if( $sendmailc =~ /\.mc$/ ){
		# better just use the address then, since there is no guarantee that the cf file will be in the same place
		$ret .= "inet:12200\@127.0.0.1";
	}elsif( $sendmailc =~ /\.cf$/ ){
		$ret .= $sendmailc;
	}else{
		die("not a .mc and not .cf?!? Something is horribly wrong");
	}
	#Search for f-prot inside file
	print "\nModifying $sendmailc ...";
	open(CF, "<$sendmailc" ) || die("Cannot open $sendmailc for reading: $!\nPlease investigate and then rerun this script\n");
	if( scalar grep(/$Config{marker}/, <CF>)){
		print "\n\n\tPrevious F-Prot configuration section found in $sendmailc";
		print "\n\tLeaving configuration intact\n";
		close(CF);
	}else{
		close(CF);
		open(CF, ">>$sendmailc") || die("Cannot open $sendmailc for updates: $!\nPlease investigate and then rerun this script\n");
		if( $sendmailc =~ /\.mc$/ ){
			print CF $mclines;
		}else{
			print CF $cflines;
		}
		close(CF);
		print " done\n";
	}
	if( $sendmailc =~ /\.mc$/ ){
		print "\n\n\t You must now generate your sendmail.cf file and reload sendmail for the changes to take effect.\n";
	}
	return $ret;
}

sub backupFile
{
	my $srcfile = $_[0];
	print "\n\nMaking backup of $srcfile...";
	if( -e $srcfile . ".fprotbackup"){
		print " hang on\n\n\tBackup file exists... Not making new backup.\n";
	}else{
		if( ! &$copyfunction($srcfile, $srcfile . ".fprotbackup") ){
			die("Backup failed: $!\n\tPlease investigate and then re-run this script\n");
		}
		print " done\n";
	}
}

sub setupRCfiles()
{
	if($Config{edition} eq "Workstation"){
		return ;
	}
	#search for a suitable rc file for f-protd
	my $rcdir = $Config{installdir} . "/tools/rc_scripts";
	my $ending;
	if( $Config{OS} =~ /linux/i ){
		$ending = $Config{distro};
		$ending =~ tr/A-Z/a-z/;
		$ending =~ s/\s*$//;
	} else {
		$ending = $Config{OS};
	}
	my $fprc = $rcdir . "/f-protd.rc-" . $ending;
	doRCfile($fprc, "f-protd");
	if($Config{edition} eq "FileServer"){
		return;
	}
	my $smrc = $rcdir . "/scan-mail.rc." . $ending;
	doRCfile($smrc, "scan-mail");
}

sub doRCfile
{
	my ($filename, $target) = @_;
	print "\n  checking for '$filename'\n";
	if( ! -e $filename ){
		print "\n No rc-script for $target available for this platform.";
		print "\n You may find that one of the scripts in $Config{installdir}/tools/rc_scripts";
		print "\n is useable, but it's up to you to modify it to fit.\n";
		print "\n $target will NOT be started automatically at boot!\n\n";
		return;
	}
	open(SRC, $filename) || die("Cannot open $filename: $!\n Please investigate and then re-run this program\n\n");
	if( -e "/etc/init.d"){
		$target = "/etc/init.d/$target";
	}elsif( -e "/etc/rc.d"){
		$target = "/etc/rc.d/$target";
	}else{
		print "\n Not sure where this OS/distro keeps it's rc-scripts.";
		print "\n You'll have to install $filename in the correct location manually.\n";
		return;
	}
	open(RC, ">$target") || die("Cannot open $target: $!\n Please investigate and then re-run this program\n\n");
	while(<SRC>){
		my $l = $_;
		if($l =~ /^BASEPATH/){
			$l =~ s/^BASEPATH=.+$/BASEPATH=$Config{installdir}/;
		}
		print RC $l;
	}
	close(SRC);
	close(RC);
	chmod(0755, "$target") || warn("Unable to change file permissions of $target: $!\n Continuing anyway\n");
}

sub fillConfigInfo
{
	# OS, cpu arch and OS release.
	chomp(($Config{OS} = `uname -s`));
	chomp(($Config{march} = `uname -m`));
	chomp(($Config{release} = `uname -r`));
	$Config{release} =~ s/(.+?)-.*/$1/; # grab release/kernel/whatever up to the first '-' character.

	# distro
	if ($Config{OS} =~ /linux/i)
	{
		if (-e "/etc/redhat-release"){
			$Config{distro} = 'RedHat '; # The space is there for formatting reasons...
		}
		elsif (-e "/etc/SuSE-release"){
			$Config{distro} = 'SuSE ';
		}
		elsif (-e "/etc/mandrake-release"){
			$Config{distro} = 'Mandrake ';
		}
		elsif (-e "/etc/debian_version"){
			$Config{distro} = 'Debian ';
		}
		elsif (-e "/etc/slackware-version"){
			$Config{distro} = 'Slackware ';
		}
		else{
			$Config{distro} = 'Unknown ';
		}
	}
	# Where am I?
	if($0 =~ /^\./){ #invoked from current working directory
		$Config{installdir} = getcwd();
	}else{
		$Config{installdir} = $0;
		$Config{installdir} =~ s/(.+)\/.+/$1/; # throw away the last slash and everything after it
	}
	#which f-prot flavor
	if(-e $Config{installdir} . "/f-protd"){
		$Config{edition} = "FileServer";
	}
	if(-e $Config{installdir} . "/tools/scan-mail.pl" ){
		$Config{edition} = "MailServer";
	}
	#link locations (we check for failure later when it's clear which ones we need)
	$Config{bin} = findDir("/usr/local/bin", "/usr/bin", "/bin");
	$Config{sbin} = findDir("/usr/local/sbin", "/usr/sbin", "/sbin");
	$Config{man1} = findDir("/usr/local/man/man1", "/usr/share/man/man1");
	$Config{man5} = findDir("/usr/local/man/man5", "/usr/share/man/man5");
	$Config{man8} = findDir("/usr/local/man/man8", "/usr/share/man/man8");
}

sub makeSymlinks
{
	# Everyone needs workstation files
	# F-Prot
	$Config{bin} = makeLinkPathOK($Config{bin}, "F-Prot Antivirus");
	makeLink($Config{bin} . "/f-prot", $Config{installdir} . "/f-prot.sh");
	# man check-updates
	$Config{man8} = makeLinkPathOK($Config{man8}, "section 8 manuals");
	makeLink($Config{man8} . "/check-updates.pl.8", $Config{installdir} . "/man_pages/check-updates.pl.8");
	# man f-prot
	$Config{man1} = makeLinkPathOK($Config{man1}, "section 1 manuals");
	makeLink($Config{man1} . "/f-prot.1", $Config{installdir} . "/man_pages/f-prot.1");
	# man f-prot.conf
	$Config{man5} = makeLinkPathOK($Config{man5}, "section 5 manuals");
	makeLink($Config{man5} . "/f-prot.conf.5", $Config{installdir} . "/man_pages/f-prot.conf.5");
	if( $Config{edition} eq "Workstation" ){ # We're done
		return;
	}
	# add links to f-protd for file server editions
	$Config{sbin} = makeLinkPathOK($Config{sbin}, "F-Prot daemon scanner");
	makeLink($Config{sbin} . "/f-protd", $Config{installdir} . "/f-protd");
	makeLink($Config{man8} . "/f-protd.8", $Config{installdir} . "/man_pages/f-protd.8");
	makeLink($Config{man8} . "/f-prot.so.8", $Config{installdir} . "/man_pages/f-prot.so.8");
	if($Config{edition} eq "FileServer"){
		return;
	}
	makeLink($Config{man8} . "/scan-mail.pl.8", $Config{installdir} . "/man_pages/scan-mail.pl.8");
	if ($Config{OS} =~ /aix/i){
		return;
	}
	makeLink($Config{man8} . "/f-prot-milter.8", $Config{installdir} . "/man_pages/f-prot-milter.8");
	makeLink($Config{man8} . "/f-prot-qmail.8", $Config{installdir} . "/man_pages/f-prot-qmail.8");
}

sub getPathToExistingFile
{
	my ($promptfor, $fullpath, $default) = @_;
	my $cre = qr/$promptfor$/;
	my $question = "Where is your $promptfor";
	if($fullpath){
		$question .= " (full path please) ";
	}
	my $reply;
	while( 1 ){
		$reply = answerMeThis($question, $default);
		if( $fullpath && ! ($reply =~ /^\// && $reply =~ $cre) ){
			print "\nPlease specify a full path to $promptfor";
			next;
		}
		if( ! -e $reply ){
			print "\n$reply doesn't exist";
			next;
		}
		if( ! -f $reply){
			print "\n$reply is not a regular file";
			next;
		}
		last;
	}
	return $reply;
}

sub makeLink
{
	my ($linkname, $target) = @_;
	if( ! symlink($target, $linkname) ){
		# it's ok to fail if link exists and points to the correct file.
		if( int($!) == $eexist){ #File exists
			my $point = readlink($linkname);
			return if $point eq $target;
			# if it points elsewhere we should ask the user.
			print "Link $linkname already exists, pointing to $point.";
			if( (my $reply=answerMeThis("Override", "Y/n")) =~ /^y/i ){
				if( ! unlink($linkname) ){
					die("\nCannot remove $linkname: $!\n Please investigate and then re-run this script\n\n");
				}
				return makeLink($linkname, $target);
			}
			die("\n\n Installation cannot continue.\n\n Fix the above errors and then re-run this script or install manually\n\n");
		}
		die("\nCouldn't create link: $!\n\n\tPlease investigate and then re-run this script\n\n");
	}
}

sub makeLinkPathOK
{
	my ($path, $component) = @_;
	my $reply;
	while( 1 ){
		$reply = answerMeThis("Where do you want a symbolic link to '$component' to be created", $path);
		return $reply if -e $reply;
		if( $reply !~ /^(\.{1,2})?\// ){
			print "\n\t$reply doesn't look like a sane path. Please enter a usable path for the symlink\n\n";
		}else{
			my $create = answerMeThis("$reply doesn't exist. Create it", "Y/n");
			if ( $create =~ /^y/i ){
				last if( mkdir($reply, 0755) );
				print "\nCannot create $reply: $!\n";
			}
		}
	}
	return $reply
}

sub findDir
{
	foreach (@_){
		return $_ if -e $_;
	}
	return undef;
}

sub answerMeThis
{
	my ($question, $default) = @_;
	my $answer;
	printf("\n%s?: [%s]\n", $question, $default);
	printf("(Just press Enter to accept the default): ");
	chop($answer = <STDIN>);
	return $default if $answer eq "";
	die "Quitting! \n" if $answer eq "q" or $answer eq "Q";
	return $answer;
}


main();

